/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_pong.c,v 1.11 2002/04/01 11:39:33 lloydy Exp $
 *
 */
#include "monkeyircd-modules.h"

CMIRCD_MODULE (1.10, CMD_CLIENT | CMD_SERVER);

/* 
 * respond to a pong
 */
int
cmd_pong (ClientInfo c_ptr, ServerInfo s_ptr, Cmd cmd)
{
  ServerInfo dest_ptr = NULL;
  ClientInfo dest_client = NULL;
  char buffer[BUF_SIZE];
  char *dest = NULL;

  /* Use either cmd->text or cmd->params[2] */
  dest = cmd->text ? cmd->text : cmd->params[2];


  /* No Target, assume its for us */
  if (!dest)
    {
      if (c_ptr)
	{
	  c_ptr->flags &= ~USR_PINGSENT;
	}
      else
	{
	  s_ptr->flags &= ~USR_PINGSENT;
	}

      return 1;
    }

  if (lc_strncmp (dest, server_name, 30) == 0)
    {
      if (c_ptr)
	{
	  c_ptr->flags &= ~USR_PINGSENT;
	}
      else
	{
	  s_ptr->flags &= ~USR_PINGSENT;
	}

      return 1;
    }

  /* Forward to another server */
  dest_ptr = server_locate (dest);

  if (!dest_ptr)
    {
      /* no such target */
      if (s_ptr && (dest_client = client_locate (dest)))
	{
	  ircd_sprintf (buffer, BUF_SIZE, "PONG %s :%s", cmd->origin,
			cmd->params[0], dest);

	  send_origin_client (dest_client, cmd->origin, buffer);
	}
      else
	{
	  /* nope */
	  if (c_ptr)
	    {
	      ircd_sprintf (buffer, BUF_SIZE, "%s :No such server", dest);
	      send_numeric (c_ptr, get_my_server (), 402, buffer);
	    }
	}

    }
  else
    {
      /* Forward */
      ircd_sprintf (buffer, BUF_SIZE, "PONG %s %s", cmd->params[0], dest);
      send_origin_server (dest_ptr, c_ptr->nick, buffer);
    }


  /* Return */
  return 1;
}
